"""
Script to check a GridFS instance for corrupted records.
"""

import sys
import logging
import argparse

from six.moves import filter, map

import pymongo
from jaraco.ui import progress
from more_itertools.recipes import consume
from jaraco.itertools import Counter

from jaraco.mongodb import helper
from jaraco.context import ExceptionTrap


log = logging.getLogger()


def get_args():
	parser = argparse.ArgumentParser()
	parser.add_argument('--depth', default=1024,
		help="Bytes to read into each file during check",
	)
	parser.add_argument('db', type=helper.connect_gridfs)
	return parser.parse_args()


class FileChecker:
	def __init__(self, gfs, depth):
		self.gfs = gfs
		self.depth = depth

	def run(self):
		files = self.gfs.list()
		bar = progress.TargetProgressBar(len(files))
		processed_files = map(self.process, bar.iterate(files))
		errors = filter(None, processed_files)
		counter = Counter(errors)
		consume(map(self.handle_trap, counter))
		return counter

	def process(self, filename):
		file = self.gfs.get_last_version(filename)
		with ExceptionTrap(pymongo.errors.PyMongoError) as trap:
			file.read(self.depth)
		trap.filename = filename
		return trap

	def handle_trap(self, trap):
		cls, exc, tb = trap.exc_info
		log.error("Failed to read %s (%s)", trap.filename, exc)


def run():
	logging.basicConfig(stream=sys.stderr)
	args = get_args()

	checker = FileChecker(args.db, args.depth)
	counter = checker.run()

	print("Encountered", counter.count, "errors")


if __name__ == '__main__':
	run()



import bson

from jaraco.mongodb import oplog

class TestReplacer:
	def test_rename_index_op_ns(self):
		"""
		As an index operation references namespaces,
		when performing a rename operation, it's also important
		to rename the ns in the op itself.
		"""
		op = {
			'ts': bson.Timestamp(1446495808, 3),
			'ns': 'airportlocker.system.indexes',
			'op': 'i',
			'o': {
				'ns': 'airportlocker.luggage.chunks',
				'key': {'files_id': 1, 'n': 1},
				'name': 'files_id_1_n_1', 'unique': True
			},
		}

		ren = oplog.Renamer.from_specs("airportlocker=airportlocker-us")
		ren(op)

		assert op['ns'] == 'airportlocker-us.system.indexes'
		assert op['o']['ns'] == 'airportlocker-us.luggage.chunks'



"""
Helper functions to augment PyMongo
"""

import warnings

import pymongo
import gridfs


def filter_warnings():
    _filter_username_warning()


def _filter_username_warning():
    """
    Suppress warnings when passing a URI to MongoDB that includes the database
    name.
    """
    warnings.filterwarnings(
        'ignore', category=UserWarning,
        message=".*must provide a username",
        module='pymongo.mongo_client' if pymongo.version_tuple >= (2,4) else
            'pymongo.connection',
    )


def _filter_safe_deprecation():
    """
    Due to a bug in pymongo, safe must be used in some cases even while it's
    deprecated. If that's the case, call this function to suppress those
    warnings.
    """
    warnings.filterwarnings(
        'ignore', category=DeprecationWarning,
        message=".*write_concern option instead",
        module='pymongo.common',
    )


def connect(uri, factory=pymongo.MongoClient):
    """
    Use the factory to establish a connection to uri.
    """
    return factory(uri)


def connect_db(uri, default_db_name=None, **kwargs):
    """
    Use pymongo to parse a uri (possibly including database name) into
    a connected database object.

    This serves as a convenience function for the common use case where one
    wishes to get the Database object and is less concerned about the
    intermediate MongoClient object that pymongo creates (though the
    connection is always available as db.client).
    """
    filter_warnings()
    uri_p = pymongo.uri_parser.parse_uri(uri)
    db_name = uri_p['database'] or default_db_name
    if not db_name:
        raise ValueError("A database name must be supplied")
    conn = connect(uri, **kwargs)
    return conn[db_name]


def connect_gridfs(uri, db=None):
    """
    Construct a GridFS instance for a MongoDB URI.
    """
    db = db or connect_db(uri)
    collection = pymongo.uri_parser.parse_uri(uri)['collection']
    return gridfs.GridFS(db, collection=collection or 'fs')
